# coding: utf-8
from django.utils.safestring import mark_safe
from m3.actions.context import ActionContext
from m3_ext.ui.containers.containers import ExtToolbarMenu
from m3_ext.ui.containers.context_menu import ExtContextMenu
from m3_ext.ui.containers.grids import ExtGridCheckBoxSelModel
from m3_ext.ui.icons import Icons
from m3_ext.ui.menus import ExtContextMenuItem
from m3_ext.ui.misc.label import ExtLabel
from m3_ext.ui.panels.grids import ExtObjectGrid
from objectpack.tree_object_pack.ui import BaseObjectTree
from objectpack.tree_object_pack.ui import BaseTreeSelectWindow
from objectpack.ui import BaseListWindow
from objectpack.ui import ModelEditWindow

from educommon import ioc

from .models import Role


class RolesTree(BaseObjectTree):

    u"""Грид для отображения иерархии ролей.

    Отличается от обычного грида для отображения деревьев тем, что помимо
    кнопок "Новый в корне" и "Новый дочерний" имеет кнопку "Добавить в роль".
    """

    def __init__(self, *args, **kwargs):
        super(RolesTree, self).__init__(*args, **kwargs)

        # Меню "Добавить"
        self.top_bar.button_add_to_role = ExtContextMenuItem(
            text=u'Добавить в роль',
            icon_cls='add_item',
            handler='topBarAddToRole',
        )
        self.top_bar.add_menu.menu.items.append(
            self.top_bar.button_add_to_role
        )

        # Меню "Удалить"
        self.top_bar.items.remove(self.top_bar.button_delete)
        self.top_bar.button_delete_from_role = ExtContextMenuItem(
            text=u'Удалить из роли',
            icon_cls='delete_item',
            handler='topBarDeleteFromRole',
        )
        self.top_bar.button_delete = ExtContextMenuItem(
            text=u'Удалить из системы',
            icon_cls='delete_item',
            handler='topBarDelete',
        )

        menu = ExtContextMenu()
        menu.items.extend((
            self.top_bar.button_delete_from_role,
            self.top_bar.button_delete,
        ))
        self.top_bar.delete_menu = ExtToolbarMenu(
            icon_cls="delete_item",
            menu=menu,
            text=u'Удалить'
        )
        self.top_bar.items.append(self.top_bar.delete_menu)


class RolesListWindow(BaseListWindow):

    u"""Окно для отображения иерархии ролей."""

    def _init_components(self):
        super(RolesListWindow, self)._init_components()

        self.grid = RolesTree()

    def set_params(self, params):
        super(RolesListWindow, self).set_params(params)

        self.template_globals = 'rbac/roles-list-window.js'
        self.pack = params['pack']
        # ---------------------------------------------------------------------
        # Включение/отключение элементов окна в зависимости от прав доступа
        if not params['can_edit']:
            # Отключение контролов для добавления ролей
            self.grid.action_new = None

            # Отключение контролов для изменения ролей
            for control in (
                self.grid.top_bar.button_edit,
                self.grid.context_menu_row.menuitem_edit
            ):
                control.text = u'Просмотр'
                control.icon_cls = Icons.APPLICATION_VIEW_DETAIL

            # Изменение контролов для удаления ролей
            self.grid.action_delete = None
            self.grid.url_delete = None
            self.grid.top_bar.items.remove(self.grid.top_bar.delete_menu)
        # ---------------------------------------------------------------------


class RoleSelectWindow(BaseTreeSelectWindow):

    u"""Окно выбора роли, в которую будет добавлена указанная роль."""

    def _init_components(self):
        super(RoleSelectWindow, self)._init_components()

        self.label_message = ExtLabel(
            text=u'Выберите роль, в которую будет добавлена роль "{}":',
            style={'padding': '5px'},
            region='north',
        )

    def _do_layout(self):
        super(RoleSelectWindow, self)._do_layout()

        self.layout = 'border'

        self.items.insert(0, self.label_message)

    def set_params(self, params):
        super(RoleSelectWindow, self).set_params(params)

        self.title = u'Добавление одной роли в другую'

        self.grid.region = 'center'
        self.grid.action_new = None
        self.grid.action_edit = None
        self.grid.action_delete = None

        if self.grid.action_context is None:
            self.grid.action_context = ActionContext()
        self.grid.action_context.role_id = params['role'].id

        self.label_message.text = (
            mark_safe(self.label_message.text.format(params['role'].name))
        )


class RoleEditWindow(ModelEditWindow):

    u"""Окно редактирования роли."""

    model = Role

    field_fabric_params = dict(
        field_list=('name', 'description', 'can_be_assigned'),
        model_register=ioc.get('observer'),
    )

    def _init_components(self):
        super(RoleEditWindow, self)._init_components()

        self.grid__partitions = ExtObjectGrid(
            region='west',
            width='35%',
        )
        self.grid__permissions = ExtObjectGrid(
            region='center',
            width='65%',
        )

    def _do_layout(self):
        super(RoleEditWindow, self)._do_layout()

        self.items.extend((
            self.grid__partitions,
            self.grid__permissions,
        ))

    def set_params(self, params):
        super(RoleEditWindow, self).set_params(params)

        self.layout = 'border'
        self.width = 800
        self.height = 600
        self.template_globals = 'rbac/role-edit-window.js'

        self.form.region = 'north'
        self.form.height = 140
        self.form.label_width = 150
        self.form.padding = 5

        self.grid__permissions.store.auto_load = False
        self.grid__permissions.sm = ExtGridCheckBoxSelModel(check_only=True)

        params['partitions_pack'].configure_grid(self.grid__partitions)
        params['permissions_pack'].configure_grid(self.grid__permissions)

        self.permission_ids = params['permission_ids']

        if not params['can_edit']:
            self.make_read_only()
